export {b58, saltPass2seed, seed2keyPair, idSecPass2rawAll, raw2b58, idSecPass2cleanKeys, pubKey2shortKey};
// Alt deps :  import {generate_keypair} from "ecma-nacl/build/lib/signing/sign.js";
// Alt deps :  import scrypt from "ecma-nacl/build/lib/scrypt/scrypt.js";
import nacl from '../generated/vendors/nacl.mjs';
import {b58} from './basex.mjs';
import sha from '../node_modules/js-sha256/src/sha256.mjs';
const sha256 = sha();

const generateKeypair = nacl.sign.keyPair.fromSeed;
import scrypt from '../generated/vendors/scrypt.mjs';

async function idSecPass2rawAll(idSec, pass) {
	const rawSeed = await saltPass2seed(idSec, pass);
	const keyPair = seed2keyPair(rawSeed);
	return {
		seed: rawSeed,
		publicKey: keyPair.publicKey,
		secretKey: keyPair.secretKey
	};
}

function raw2b58(raws) {
	const result = {};
	for (const r in raws) result[r] = b58.encode(raws[r]);
	return result;
}

async function idSecPass2cleanKeys(idSec, pass) {
	const raw = await idSecPass2rawAll(idSec, pass);
	return Object.assign(raw2b58(raw), {idSec, password: pass});
}

function seed2keyPair(seed) {
	return generateKeypair(seed);
}

async function saltPass2seed(idSec, pass) {
	const options = {
		logN: 12,
		r: 16,
		p: 1,
		// Default: dkLen: 32,
		encoding: 'binary'
	};
	return scrypt(pass.normalize('NFKC'), idSec.normalize('NFKC'), options);
}

function pubKey2shortKey(pubkey) {
	const checksum = b58.encode(sha256.digest(sha256.digest(b58.decode(pubkey))));
	const pubKeyBegin = pubkey.substr(0, 4);
	const pubKeyEnd = pubkey.substr(-4, 4);
	const hashBegin = checksum.substr(0, 3);
	return `${pubKeyBegin}…${pubKeyEnd}:${hashBegin}`;
}
