import test from 'ava';
import * as app from './crypto.mjs';

const idSec = 'a';
const mdp = 'b';
// Base58
const pubKey = 'AoxVA41dGL2s4ogMNdbCw3FFYjFo5FPK36LuiW1tjGbG';
const secretKey = '3ZsmZhnRv137dS1s7Q3jFGKLTDyhkwguPHfnWBxzDCTTHKWGnYw9zBk3gcCUJCc72TEUuyzM7cqpo7c5LYhs1Qtv';
const seed = '9eADqX8V6VcPdJCHCVYiE1Vnift9nFNrvr9aTaXA5RJc';

test('b58 should decode/encode well', t => {
	t.is(app.b58.encode(app.b58.decode(pubKey)), pubKey);
});
test('saltPass2seed should convert salt & password to seed with scrypt', async t => {
	t.is(app.b58.encode(await app.saltPass2seed(idSec, mdp)), seed);
});
test('seed2keyPair should generate public and private key nacl/sodium way.', t => {
	const rawSeed = app.b58.decode(seed);
	const rawKeyPair = app.seed2keyPair(rawSeed);
	t.is(app.b58.encode(rawKeyPair.publicKey), pubKey);
	t.is(app.b58.encode(rawKeyPair.secretKey), secretKey);
});
test('idSecPass2cleanKeys should output clean base58 keys and seed', async t => {
	const r = await app.idSecPass2cleanKeys(idSec, mdp);
	t.is(r.publicKey, pubKey);
	t.is(r.secretKey, secretKey);
	t.is(r.seed, seed);
	t.is(r.idSec, idSec);
	t.is(r.password, mdp);
});
test('pubKey2shortKey match D2me…wRaU:76W', t => {
	const pubKey = 'D2meevcAHFTS2gQMvmRW5Hzi25jDdikk4nC4u1FkwRaU';
	const shortKey = 'D2me…wRaU:76W';
	t.is(app.pubKey2shortKey(pubKey), shortKey);
});
test('pubKey2shortKey match RML1…zvSY:3k4', t => {
	const pubKey = 'RML12butzV3xZmkWnNAmRwuepKPYvzQ4euHwhHhzvSY';
	const shortKey = 'RML1…zvSY:3k4';
	t.is(app.pubKey2shortKey(pubKey), shortKey);
});
